'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.argsToFindOptions = argsToFindOptions;
exports.buildQuery = buildQuery;
exports.buildCount = buildCount;
exports.mapCountToResultSet = mapCountToResultSet;

var _thinkyExportSchema = require('thinky-export-schema');

var _thinkyExportSchema2 = _interopRequireDefault(_thinkyExportSchema);

var _resolver = require('./resolver');

var _node = require('./node');

var _lodash = require('lodash');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Args to find options
 *
 * @param args
 * @param model
 * @param opts
 * @returns {{}}
 */
function argsToFindOptions(args, attributes, model) {
  var _ref = arguments.length <= 3 || arguments[3] === undefined ? { maxLimit: 50 } : arguments[3];

  var maxLimit = _ref.maxLimit;
  var requestedFields = _ref.requestedFields;

  var result = {
    filter: {},
    attributes: [],
    limit: undefined,
    skip: undefined,
    orderBy: undefined
  },
      modelSchema = (0, _thinkyExportSchema2.default)(model),
      modelAttributes = Object.keys(modelSchema.fields).concat('id'),
      modelRelations = Object.keys(modelSchema.relationships);

  if (args) {
    Object.keys(args).forEach(function (key) {
      if (modelAttributes.indexOf(key) !== -1) {
        result.filter = result.filter || {};
        result.filter[key] = args[key];
      }

      // Limit arg
      if (key === 'offset' && args[key]) {
        result.offset = parseInt(args[key], 10);
      }

      if (key === 'skip' && args[key]) {
        result.index = parseInt(args[key], 10);
      }

      if (key === 'order' && args[key]) {
        if (args[key].indexOf('reverse:') === 0) {
          result.orderBy = [args[key].substring(8), 'DESC'];
        } else {
          result.orderBy = [args[key], 'ASC'];
        }
      }
    });

    attributes = Object.keys(attributes).filter(function (field) {
      return modelRelations.indexOf(field) === -1 && modelSchema.fields.hasOwnProperty(field) && modelSchema.fields[field] !== 'Virtual';
    }).concat(['id']);

    if (Array.isArray(requestedFields)) {
      attributes = attributes.concat(requestedFields);
    }

    result.attributes = (0, _lodash.uniq)(attributes);

    // Set the maxLimit that can be set
    // for an offset or use that by default
    // if not explicitly disabled
    if (maxLimit !== false) {
      if (!result.offset) {
        result.offset = maxLimit || 100;
      }

      if (result.offset > maxLimit) {
        result.offset = maxLimit;
      }
    }

    return result;
  }
}

/**
 * Build query
 *
 * @param seq
 * @param args
 * @param thinky
 * @param options
 * @returns {*}
 */
function buildQuery(seq, args, thinky) {
  var Query = seq;

  // Developer can overwrite query per node
  if (typeof args.query === 'function') {
    Query = args.query(seq, args, thinky);
  } else {

    // NOTE: selecting only necessary fields will gather a bit of performance
    // through your queries, BUT the drawback is that we might
    // add another DB query for sub-sequentials GraphQL requests
    // when asking for the the same resource. We can't gurantee all the fields
    // are available so we don't use Dataloader cache to return the results
    if (args.requestedFields) {
      if (Array.isArray(args.attributes)) {
        Query = seq.withFields(args.attributes);
      } else if ((0, _lodash.isObject)(args.attributes)) {
        Query = seq.withFields(args.attributes);
      }
    }

    if (args.filter && args.filterQuery && Object.keys(args.filter).length > 0) {

      Object.keys(args.filter).forEach(function (fieldName) {
        if ((0, _lodash.isFunction)(args.filter[fieldName])) {
          Query = Query.filter(args.filter[fieldName]);
          delete args.filter[fieldName];
        }
      });

      Query = Query.filter(args.filter);
    }

    if (args.orderBy && args.orderBy[1] === 'DESC') {
      Query = Query.orderBy(thinky.r.desc(args.orderBy[0]));
    } else if (args.orderBy && args.orderBy[0]) {
      Query = Query.orderBy(args.orderBy[0]);
    }

    if (args.offset) {
      Query = Query.slice(args.index, args.offset);
    }
  }

  return Query;
}

/**
 * Build Count
 * @param model
 * @param relatedIds
 * @param FK
 * @param opts
 * @returns {*|void|Promise}
 */
function buildCount(model, relatedIds, FK, opts) {
  var thinky = model._thinky;
  var r = thinky.r;
  opts.offset = false;
  opts.orderBy = false;

  if (relatedIds.length > 0 && FK) {
    var _seq = model.getAll(r.args((0, _lodash.uniq)(relatedIds)), { index: FK }).withFields(['id', FK].concat(Object.keys(opts.filter)));

    _seq = buildQuery(_seq, opts, thinky);

    return _seq.group(FK).ungroup().merge(function (selectionSet) {
      return { fullCount: selectionSet('reduction').count() };
    }).run();
  }

  var seq = buildQuery(model, opts, thinky);

  return seq.count().execute();
}

/**
 * Map count to result set
 * @param resultSet
 * @param countResults
 * @param FK
 * @returns {*}
 */
function mapCountToResultSet(resultSet, countResults, FK) {
  return resultSet.map(function (row) {
    if (Array.isArray(row)) {
      return row.map(function (innerRow) {
        var findCount = (0, _lodash.find)(countResults, { group: innerRow[FK] }) || { fullCount: 0 };
        innerRow.fullCount = findCount.fullCount;
        return innerRow;
      });
    } else {
      var findCount = (0, _lodash.find)(countResults, { group: row[FK] }) || { fullCount: 0 };
      row.fullCount = findCount.fullCount;
      return row;
    }
  });
}